// Interface oriente Delphi pour le dveloppement de plugin pour PhotoFiltre
// Base sur l'interface de premier niveau

// Version 1.6 / Avril 2005 / Copyright Antonio Da Cruz
// Merci de ne pas modifier les sources sans mon autorisation !

unit Plugin;

interface

uses
  Windows, Classes, Graphics, SysUtils, Forms, Dialogs, PlgInt;

const
  RGBColorCount = 256 * 256 * 256; // 16 millions de couleurs
  
var
  PluginErrorMsg: string = 'Erreur pendant l''excution du module !';

type
  PApplication = ^TApplication;
  PPluginLanguage = ^TPluginLanguage;
  PPluginHeader = ^TPluginHeader;
  PPluginStruct = ^TPluginStruct;

  // pour SetHeaderPluginType
  TPluginType = (ptFilter, ptImage, ptFileImport, ptFileExport, ptTool);

  TSelectionOption = (soAuto, soNone); // SetSelBitmap
  TImageOption = (ioCreate, ioFile); // NewImage
  TDialogOption = (doModal, doPreview, doColorPicker); // ShowForm
  TColorMode = (cmIndexed, cmRGB, cmRGBLayered, cmRGBA); // GetImageColorode
  TAlphaRange = 0..100; // pour AlphaBlend

  // pour l'activation du menu (SetHeaderMenuOptions)
  TMenuOption = (moRGBColors, moIndexedColors, moTransparent, moNoTransparent,
    moSelected, moNotSelected, moNeedImage, moNeedMoreThanOneImage,
    moLayered, moNotLayered, moRGBAColors, moNotRGBAColors,
    moImageJpeg, moImageGif, moImagePng, moImageBmp, moImageTiff,
    moImagePfs, moJpegExifData);
  TMenuOptions = set of TMenuOption;

  // pour GetImgInfosByName
  TImageInfos = record
    FileName: string;
    Width: Integer;
    Height: Integer;
    ColorCount: Integer;
    TransparentColor: TColor;
  end;

  // pour SetApplicationState
  TApplicationState = (asMinimize, asRestore, asMaximize, asHideForms,
    asShowForms);

  EPluginException = class(Exception);

var
  // pointeurs vers les variables globales dans PlgInt
  PtrPlgLanguage: PPluginLanguage;
  PtrPlgHeader: PPluginHeader;
  PtrPlgStruct: PPluginStruct;

// rcupration de la langue en cours
function GetLanguageCode: string;
function GetLanguageStr: string;

// initialisation de l'entte
procedure SetHeaderAuthor(Author: string);
procedure SetHeaderCaption(Caption: string);
procedure SetHeaderComment(Comment: string);
procedure SetHeaderMenuOptions(Options: TMenuOptions);
procedure SetHeaderPluginType(PluginType: TPluginType);
procedure SetHeaderVersion(Version: string);

// ces fonctions sont  utiliser uniquement dans la fonction Excute
procedure BeginFilter(Msg: string = ''; ChangeCursor: Boolean = True);
procedure EndFilter;

function MainHandle: Integer;
function ImageExists: Boolean;
function XP_ThemesEnabled: Boolean;
function GetImgRect: TRect;
function GetImgHeight: Integer;
function GetImgWidth: Integer;

function IsSelected: Boolean;
function GetSelRect: TRect;
function GetSelHeight: Integer;
function GetSelWidth: Integer;

function GetBitmap(Bitmap: TBitmap): Boolean;
function SetBitmap(Bitmap: TBitmap): Boolean;
function GetSelBitmap(Bitmap: TBitmap): Boolean;
function SetSelBitmap(Bitmap: TBitmap;
  Option: TSelectionOption = soAuto): Boolean;
function GetSelMask(Bitmap: TBitmap): Boolean;

function GetColorCount: Integer;
function SetColorCount(ColorCount: Integer): Integer;
function GetTransparentColor: TColor;
function SetTransparentColor(Color: TColor): Boolean;

function SetAlphaBlend(Alpha: TAlphaRange): Boolean;
function DrawSelBorder(Width: Integer; Color: TColor;
  Antialias: Boolean): Boolean;

function RestoreUndoBitmap: Boolean;
procedure ResetSaveMenu;

function GetImgFileName: string;
function SetImgFileName(FileName: string): Boolean;

function EnumImages(EnumImageProc: TEnumImageProc): Boolean;
function GetBitmapByName(Bitmap: TBitmap; FileName: string): Boolean;
function GetImgInfosByName(FileName: string;
  var ImageInfos: TImageInfos): Boolean;

function GetThumb(Bitmap: TBitmap; BkColor: TColor; BorderSize,
  ShadowSize: Integer; ShadowBkColor: TColor; var R: TRect): Boolean;
function GetThumbByName(Bitmap: TBitmap; FileName: string; BkColor: TColor;
  BorderSize, ShadowSize: Integer; ShadowBkColor: TColor;
  var R: TRect): Boolean;

function NewImage(Bitmap: TBitmap; FileName: string; Option: TImageOption;
  ColorCount: Integer = RGBColorCount; TransColor: TColor = clNone): Boolean;

procedure ShowForm(Form: TForm; Option: TDialogOption;
  MainFormCenter: Boolean = True; CloseWaiting: Boolean = True);

function GetExePath: string;
function GetIniFile: string;
function GetOpenPath: string;
function GetPluginPath: string;
function GetSavePath: string;

// Compatibilit de niveau >= 2
procedure PrepareDialog(Handle: HWND; Option: TDialogOption;
  MainFormCenter, CloseWaiting: Boolean);

function CreateDBmp(pImg: PPlgImage): Boolean;
function FreeDBmp(pImg: PPlgImage): Boolean;
function ReleaseHandleDBmp(pImg: PPlgImage): Boolean;
function GetScanLineDBmp(pImg: PPlgImage; Row: Integer): Pointer;

// Compatibilit de niveau >= 3
function GetSelSelMask(Bitmap: TBitmap): Boolean;
function GetImageDPI: Integer;
function SetImageDPI(DPI: Integer): Integer;
function ResetUndoArea(R: TRect): TRect;
function SetSelMask(Bitmap: TBitmap; Flag: Integer): Boolean;
function SetApplicationState(State: TApplicationState): Integer;
function GetSelShape(Points: PPoint): Integer;

// Compatibilit de niveau >= 4 (PhotoFiltre Studio)
function IsStudioVersion: Boolean;
function GetLayerRect: TRect;
function GetLayerHeight: Integer;
function GetLayerWidth: Integer;
function GetLayerPos: TPoint;

function GetFlatImage(Bitmap: TBitmap): Boolean;
function SetFlatImage(Bitmap: TBitmap): Boolean;

function GetLayerBitmap(Bitmap: TBitmap): Boolean;
function SetLayerBitmap(Bitmap: TBitmap): Boolean;
function GetLayerAlpha(Bitmap: TBitmap): Boolean;
function SetLayerAlpha(Bitmap: TBitmap): Boolean;
function GetSelAlpha(Bitmap: TBitmap): Boolean;
function SetSelAlpha(Bitmap: TBitmap): Boolean;

procedure DisableLayersPainting(Disable: Boolean);
function GetImageColorMode: TColorMode;

// Rcupre la compatibilit max supporte par PhotoFiltre
function GetCompatibilityCap: Integer;

// Gestion des exceptions
function CheckPluginFunc(RetVal: Boolean): Boolean;
procedure ShowPluginErrorMsg;

implementation

// rcupration de la langue en cours
// ----------------------------------

function GetLanguageCode: string;
begin
  Result := PtrPlgLanguage.Code[0] + PtrPlgLanguage.Code[1];
end;

function GetLanguageStr: string;
begin
  Result := PtrPlgLanguage.Language;
end;

// initialisation de l'entte
// --------------------------

procedure SetHeaderCaption(Caption: string);
begin
  StrPLCopy(PtrPlgHeader.Caption, Caption, MaxShortStr);
end;

procedure SetHeaderVersion(Version: string);
begin
  StrPLCopy(PtrPlgHeader.Version, Version, MaxShortStr);
end;

procedure SetHeaderPluginType(PluginType: TPluginType);
const
  Flags: array[TPluginType] of Integer = (PT_FILTER, PT_IMAGE,
    PT_FILE_IMPORT, PT_FILE_EXPORT, PT_TOOL);
begin
  PtrPlgHeader.PluginType := Flags[PluginType];
end;

procedure SetHeaderAuthor(Author: string);
begin
  StrPLCopy(PtrPlgHeader.Author, Author, MaxShortStr);
end;

procedure SetHeaderComment(Comment: string);
begin
  StrPLCopy(PtrPlgHeader.Comment, Comment, MaxLongStr);
end;

procedure SetHeaderMenuOptions(Options: TMenuOptions);
var
  Flags: Integer;
begin
  Flags := 0;
  if moRGBColors in Options then Flags := Flags or MF_RGB_COLORS;
  if moIndexedColors in Options then Flags := Flags or MF_INDEXED_COLORS;
  if moTransparent in Options then Flags := Flags or MF_TRANSPARENT;
  if moNoTransparent in Options then Flags := Flags or MF_NOT_TRANSPARENT;
  if moSelected in Options then Flags := Flags or MF_SELECTED;
  if moNotSelected in Options then Flags := Flags or MF_NOT_SELECTED;
  if moNeedImage in Options then Flags := Flags or MF_NEED_IMAGE;
  if moNeedMoreThanOneImage in Options then
    Flags := Flags or MF_NEED_MORE_THAN_ONE_IMAGE;
  if moLayered in Options then Flags := Flags or MF_LAYERED;
  if moNotLayered in Options then Flags := Flags or MF_NOT_LAYERED;
  if moRGBAColors in Options then Flags := Flags or MF_RGBA_COLORS;
  if moNotRGBAColors in Options then Flags := Flags or MF_NOT_RGBA_COLORS;
  if moImageJpeg in Options then Flags := Flags or MF_IMAGE_JPEG;
  if moImageGif in Options then Flags := Flags or MF_IMAGE_GIF;
  if moImagePng in Options then Flags := Flags or MF_IMAGE_PNG;
  if moImageBmp in Options then Flags := Flags or MF_IMAGE_BMP;
  if moImageTiff in Options then Flags := Flags or MF_IMAGE_TIFF;
  if moImagePfs in Options then Flags := Flags or MF_IMAGE_PFS;
  if moJpegExifData in Options then Flags := Flags or MF_JPEG_EXIF_DATA;  
  PtrPlgHeader.Flags := PtrPlgHeader.Flags or Flags;
end;

// ces fonctions sont  utiliser uniquement dans la fonction Excute
//-----------------------------------------------------------------

function MainHandle: Integer;
begin
  Result := PtrPlgStruct.MainHandle;
end;

function ImageExists: Boolean;
begin
  Result := PtrPlgStruct.ImageExists;
end;

function IsSelected: Boolean;
begin
  Result := PtrPlgStruct.Selected;
end;

function GetSelRect: TRect;
begin
  Result := PtrPlgStruct.SelRect;
end;

function GetSelWidth: Integer;
begin
  with PtrPlgStruct.SelRect do
    Result := Right - Left;
end;

function GetSelHeight: Integer;
begin
  with PtrPlgStruct.SelRect do
    Result := Bottom - Top;
end;

function GetImgRect: TRect;
begin
  Result := PtrPlgStruct.ImgRect;
end;

function GetImgWidth: Integer;
begin
  with PtrPlgStruct.ImgRect do
    Result := Right - Left;
end;

function GetImgHeight: Integer;
begin
  with PtrPlgStruct.ImgRect do
    Result := Bottom - Top;
end;

function XP_ThemesEnabled: Boolean;
begin
  Result := PtrPlgStruct.XP_ThemesEnabled;
end;

procedure BeginFilter(Msg: string; ChangeCursor: Boolean);
begin
  PtrPlgStruct.BeginFilter(PChar(Msg), ChangeCursor);
end;

procedure EndFilter;
begin
  PtrPlgStruct.EndFilter;
end;

function GetBitmap(Bitmap: TBitmap): Boolean;
begin
  Result := PtrPlgStruct.GetBitmap(Bitmap);
end;

function SetBitmap(Bitmap: TBitmap): Boolean;
begin
  Result := PtrPlgStruct.SetBitmap(Bitmap);
end;

function RestoreUndoBitmap: Boolean;
begin
  Result := PtrPlgStruct.SetBitmap(nil);
end;

function GetSelBitmap(Bitmap: TBitmap): Boolean;
begin
  Result := PtrPlgStruct.GetSelBitmap(Bitmap);
end;

function SetSelBitmap(Bitmap: TBitmap; Option: TSelectionOption): Boolean;
const
  Flags: array[TSelectionOption] of Integer = (SF_AUTO, SF_NONE);
begin
  Result := PtrPlgStruct.SetSelBitmap(Bitmap, Flags[Option]);
end;

function GetTransparentColor: TColor;
begin
  Result := PtrPlgStruct.GetTransparentColor;
end;

function SetTransparentColor(Color: TColor): Boolean;
begin
  Result := PtrPlgStruct.SetTransparentColor(Color);
end;

function GetColorCount: Integer;
begin
  Result := PtrPlgStruct.GetColorCount;
end;

function SetColorCount(ColorCount: Integer): Integer;
begin
  Result := PtrPlgStruct.SetColorCount(ColorCount);
end;

function NewImage(Bitmap: TBitmap; FileName: string; Option: TImageOption;
  ColorCount: Integer; TransColor: TColor): Boolean;
const
  Flags: array[TImageOption] of Integer = (IF_CREATE, IF_FILE);
begin
  Result := PtrPlgStruct.NewImage(Bitmap, PChar(FileName), Flags[Option],
    ColorCount, TransColor);
end;

procedure ShowForm(Form: TForm; Option: TDialogOption;
  MainFormCenter, CloseWaiting: Boolean);
const
  Flags: array[TDialogOption] of Integer = (DM_MODAL, DM_PREVIEW,
    DM_COLOR_PICKER);
begin
  PtrPlgStruct.ShowForm(Form, Flags[Option], MainFormCenter, CloseWaiting);
end;

function SetAlphaBlend(Alpha: TAlphaRange): Boolean;
begin
  Result := PtrPlgStruct.SetAlphaBlend(Alpha);
end;

function DrawSelBorder(Width: Integer; Color: TColor;
  Antialias: Boolean): Boolean;
begin
  Result := PtrPlgStruct.DrawSelBorder(Width, Color, Antialias);
end;

function GetSelMask(Bitmap: TBitmap): Boolean;
begin
  Result := PtrPlgStruct.GetSelMask(Bitmap);
end;

procedure ResetSaveMenu;
begin
  PtrPlgStruct.ResetSaveMenu;
end;

function GetImgFileName: string;
var
  Buffer: array[0..MaxLongStr] of Char;
begin
  Result := PtrPlgStruct.GetImgFileName(Buffer, MaxLongStr);
end;

function GetExePath: string;
var
  Buffer: array[0..MaxLongStr] of Char;
begin
  Result := PtrPlgStruct.GetExePath(Buffer, MaxLongStr);
end;

function GetOpenPath: string;
var
  Buffer: array[0..MaxLongStr] of Char;
begin
  Result := PtrPlgStruct.GetOpenPath(Buffer, MaxLongStr);
end;

function GetSavePath: string;
var
  Buffer: array[0..MaxLongStr] of Char;
begin
  Result := PtrPlgStruct.GetSavePath(Buffer, MaxLongStr);
end;

function GetPluginPath: string;
var
  Buffer: array[0..MaxLongStr] of Char;
begin
  Result := PtrPlgStruct.GetPluginPath(Buffer, MaxLongStr);
end;

function GetIniFile: string;
var
  Buffer: array[0..MaxLongStr] of Char;
begin
  Result := PtrPlgStruct.GetIniFile(Buffer, MaxLongStr);
end;

function SetImgFileName(FileName: string): Boolean;
begin
  Result := PtrPlgStruct.SetImgFileName(PChar(FileName));
end;

function GetBitmapByName(Bitmap: TBitmap; FileName: string): Boolean;
begin
  Result := PtrPlgStruct.GetBitmapByName(Bitmap, PChar(FileName));
end;

function GetThumb(Bitmap: TBitmap; BkColor: TColor; BorderSize,
  ShadowSize: Integer; ShadowBkColor: TColor; var R: TRect): Boolean;
begin
  Result := PtrPlgStruct.GetThumb(Bitmap, BkColor, BorderSize, ShadowSize,
    ShadowBkColor, R);
end;

function GetThumbByName(Bitmap: TBitmap; FileName: string; BkColor: TColor;
  BorderSize, ShadowSize: Integer; ShadowBkColor: TColor;
  var R: TRect): Boolean;
begin
  Result := PtrPlgStruct.GetThumbByName(Bitmap, PChar(FileName), BkColor,
    BorderSize, ShadowSize, ShadowBkColor, R);
end;

function EnumImages(EnumImageProc: TEnumImageProc): Boolean;
begin
  Result := PtrPlgStruct.EnumImages(EnumImageProc);
end;

// plus pratique que EnumImages

var
  _FileName: string;
  _PImageInfos: ^TImageInfos;
  _ImageFinded: Boolean;

function _EnumImageProc(pFileName: PChar; Width, Height,
  NbColors: Integer; TransColor: TColor): LongBool; stdcall;
begin
  if pFileName = _FileName then
  begin
    _ImageFinded := True;
    _PImageInfos.FileName := pFileName;
    _PImageInfos.Width := Width;
    _PImageInfos.Height := Height;
    _PImageInfos.ColorCount := NbColors;
    _PImageInfos.TransparentColor := TransColor;
  end;
  Result := True;
end;

function GetImgInfosByName(FileName: string;
  var ImageInfos: TImageInfos): Boolean;
begin
  _ImageFinded := False;
  _FileName := FileName;
  _PImageInfos := @ImageInfos;
  Result := EnumImages(_EnumImageProc) and _ImageFinded;
end;

// Compatibilit de niveau >= 2
// ----------------------------

procedure PrepareDialog(Handle: HWND; Option: TDialogOption;
  MainFormCenter, CloseWaiting: Boolean);
const
  Flags: array[TDialogOption] of Integer = (DM_MODAL, DM_PREVIEW,
    DM_COLOR_PICKER);
begin
  PtrPlgStruct.PrepareDialog(Handle, Flags[Option], MainFormCenter,
    CloseWaiting);
end;

function CreateDBmp(pImg: PPlgImage): Boolean;
begin
  Result := PtrPlgStruct.CreateDBmp(pImg) <> 0;
end;

function FreeDBmp(pImg: PPlgImage): Boolean;
begin
  Result := PtrPlgStruct.FreeDBmp(pImg) <> 0;
end;

function ReleaseHandleDBmp(pImg: PPlgImage): Boolean;
begin
  Result := PtrPlgStruct.ReleaseHandleDBmp(pImg) <> 0;
end;

function GetScanLineDBmp(pImg: PPlgImage; Row: Integer): Pointer;
begin
  Result := PtrPlgStruct.GetScanlineDBmp(pImg, Row);
end;

// Compatibilit de niveau >= 3
// ----------------------------

function GetSelSelMask(Bitmap: TBitmap): Boolean;
begin
  Result := PtrPlgStruct.GetSelSelMask(Bitmap);
end;

function GetImageDPI: Integer;
begin
  Result := PtrPlgStruct.GetImageDPI;
end;

function SetImageDPI(DPI: Integer): Integer;
begin
  Result := PtrPlgStruct.SetImageDPI(DPI);
end;

function SetSelMask(Bitmap: TBitmap; Flag: Integer): Boolean;
begin
  Result := PtrPlgStruct.SetSelMask(Bitmap, Flag);
end;

function ResetUndoArea(R: TRect): TRect;
begin
  Result := PtrPlgStruct.ResetUndoArea(R);
end;

function SetApplicationState(State: TApplicationState): Integer;
begin
  Result := PtrPlgStruct.SetApplicationState(Ord(State));
end;

function GetSelShape(Points: PPoint): Integer;
begin
  Result := PtrPlgStruct.GetSelShape(Points);
end;

// Compatibilit de niveau >= 4 (PhotoFiltre Studio)
// -------------------------------------------------

// plus cohrent que GetBitamp et SetBitmap

function GetFlatImage(Bitmap: TBitmap): Boolean;
begin
  Result := PtrPlgStruct.GetBitmap(Bitmap);
end;

function SetFlatImage(Bitmap: TBitmap): Boolean;
begin
  Result := PtrPlgStruct.SetBitmap(Bitmap);
end;

function IsStudioVersion: Boolean;
begin
  Result := Assigned(PtrPlgStruct.GetLayerRect);
end;

function GetLayerRect: TRect;
begin
  Result := PtrPlgStruct.GetLayerRect;
end;

function GetLayerHeight: Integer;
var
  R: TRect;
begin
  R := PtrPlgStruct.GetLayerRect;
  Result := R.Bottom - R.Top;
end;

function GetLayerWidth: Integer;
var
  R: TRect;
begin
  R := PtrPlgStruct.GetLayerRect;
  Result := R.Right - R.Left;
end;

function GetLayerPos: TPoint;
begin
  Result := PtrPlgStruct.GetLayerPos;
end;

function GetLayerBitmap(Bitmap: TBitmap): Boolean;
begin
  Result := PtrPlgStruct.GetLayerBitmap(Bitmap);
end;

function SetLayerBitmap(Bitmap: TBitmap): Boolean;
begin
  Result := PtrPlgStruct.SetLayerBitmap(Bitmap);
end;

function GetLayerAlpha(Bitmap: TBitmap): Boolean;
begin
  Result := PtrPlgStruct.GetLayerAlpha(Bitmap);
end;

function SetLayerAlpha(Bitmap: TBitmap): Boolean;
begin
  Result := PtrPlgStruct.SetLayerAlpha(Bitmap);
end;

function GetSelAlpha(Bitmap: TBitmap): Boolean;
begin
  Result := PtrPlgStruct.GetSelAlpha(Bitmap);
end;

function SetSelAlpha(Bitmap: TBitmap): Boolean;
begin
  Result := PtrPlgStruct.SetSelAlpha(Bitmap);
end;

procedure DisableLayersPainting(Disable: Boolean);
begin
  PtrPlgStruct.DisableLayersPainting(Disable);
end;

function GetImageColorMode: TColorMode;
begin
  Result := TColorMode(PtrPlgStruct.GetImageColorMode);
end;

// Rcupre la compatibilit max supporte par PhotoFiltre
function GetCompatibilityCap: Integer;
begin
  Result := 1;
  if Assigned(PtrPlgStruct.PrepareDialog) then Inc(Result) // 2
  else Exit;
  if Assigned(PtrPlgStruct.GetSelSelMask) then Inc(Result) // 3
  else Exit;
  if Assigned(PtrPlgStruct.GetLayerRect) then Inc(Result) // 4
  else Exit;
end;

// Gestion des exceptions
// ----------------------

function CheckPluginFunc(RetVal: Boolean): Boolean;
begin
  if not RetVal then raise EPluginException.Create(PluginErrorMsg);
  Result := RetVal;
end;

procedure ShowPluginErrorMsg;
begin
  MessageDlg(PluginErrorMsg, mtError, [mbOk], 0);
end;

initialization
  PtrPlgLanguage := @PluginLanguage;
  PtrPlgHeader := @PluginHeader;
  PtrPlgStruct := @PluginStruct;
  ZeroMemory(PtrPlgStruct, SizeOf(TPluginLanguage));
  ZeroMemory(PtrPlgHeader, SizeOf(TPluginHeader));
  ZeroMemory(PtrPlgStruct, SizeOf(TPluginStruct));
end.
